@@ -429,7 +429,7 @@ virtualenv_module <- function(python) {
         # `apt install python3` makes an importable venv module, but not one
         # capable of actually creating a venv unless python3-venv is installed.
         # if python3-venv is not installed, move on and maybe discover virtualenv.
-        if (!any(grepl("^python[0-9.]*-venv$", system("dpkg -l", intern = TRUE))))
+        if (!any(grepl("\\bpython[0-9.]*-venv\\b", system("dpkg -l", intern = TRUE))))
           next
       }
       return(module)
@@ -519,14 +519,18 @@ virtualenv_starter <- function(version = NULL, all = FALSE) {
     # - nested directory of python installations (presumably nested by version and arch)
 
       suffix <- if (is_windows())
-        #e.g, /3.9.4/python.exe, /3.9.4/x64/python.exe
-        c("/*/python*.exe", "/*/*/python*.exe")
+        # <base_prefix>   /3.9.4/python.exe, /3.9.4/x64/python.exe
+        c("/python*.exe", "/*/python*.exe", "/*/*/python*.exe")
       else {
-        #/3.9.4/bin/python, /3.9.4/x64/bin/python
-        c("/*/bin/python*", "/*/*/bin/python*")
+        # <base_prefix>   /3.9.4/bin/python, /3.9.4/x64/bin/python
+        c("/bin/python*", "/*/bin/python*", "/*/*/bin/python*")
       }
-      glob <- paste0(normalizePath(glob, winslash = "/", mustWork = FALSE),
-                     suffix)
+
+      # retain the original value of `glob`, for the case it is a path
+      # to a python binary (and not a path to a directory of python installations).
+      append(glob) <-
+        paste0(normalizePath(glob, winslash = "/", mustWork = FALSE),
+               suffix)
     }
 
     p <- unique(normalizePath(Sys.glob(glob), winslash = "/"))
@@ -619,6 +623,16 @@ virtualenv_starter <- function(version = NULL, all = FALSE) {
   if(!is.na(tool_cache_dir <- Sys.getenv("RUNNER_TOOL_CACHE", NA)))
     find_starters(paste0(tool_cache_dir, "/Python"))
 
+  # If user pointed reticulate at a python via RETICULATE_PYTHON, use_python(),
+  # or similar, include that python in the list of starters found. Note that
+  # find_starters() / normalizePath() will follow the link of python binary in a
+  # venv to resolve the starter used.
+  find_starters(Sys.getenv("RETICULATE_PYTHON"))
+  find_starters(Sys.getenv("RETICULATE_PYTHON_ENV"))
+  find_starters(py_exe())
+  find_starters(Sys.which("python3"))
+  find_starters(Sys.which("python"))
+
   # if specific version requested, filter for that.
   if (!is.null(version)) {
     for (check in as_version_constraint_checkers(version)) {
@@ -696,7 +710,7 @@ can_be_virtualenv_starter <- function(python) {
 
   py2_modules <- c("pip", "virtualenv")
   py3_modules <- c("pip", "venv")
-  modules <- ifelse(version < 3, py2_modules, py3_modules)
+  modules <- ifelse(version < "3", py2_modules, py3_modules)
 
   # ensure these modules are available
   if (!python_has_modules(python, modules))
